'-- Hoertest.BAS -- 1288 Bytes flash --------------------------- >>fst'13<< -
' - Testen des Hrvermgens bei hohen Frequenzen
' - Gibt stufenweise sinkenden Ton von 20 - 5 kHz aus und
'   zeigt die Frequenz bei Tastendruck an
' - Programmiersprache: Bascom AVR - http://www.mcselec.com
' - Erforderliche Hardware:
'   + Aufgebaute Platine "Pollin Spiel 1" - http://www.Pollin.de
'   + AVR ISP Programmer
'-- V 20130118 ------------------------------- http://www.FrankSteinberg.de -


'*** ATtiny konfigurieren wie Auslieferungszustand:
$regfile = "ATtiny2313.dat"  'Pollin-Spiel hat einen ATtiny2313
$crystal = 1000000           'interner Oszillator auf 1 MHz

'*** Verschiedene Eintellungen:
Ddrd = &B01100000            'D5 (Schallwandler), D6 (LCD-Spannung) als Ausgang; Rest als Eingang
Dim Status As Byte           'Statusflag fr Programmablauf
Dim Tasten As Byte           'Variable fr Zustand der 5 Tasten

'*** LCD konfigurieren:
Portb.4 = 1                  'Kontrastspannung ein
Portd.6 = 1                  'Versorgungsspannung ein
Config Lcdbus = 4            '4-Bit Modus
Config Lcd = 16 * 1a         '8*2 wird vom Compiler nicht akzeptiert
Config Lcdpin = Pin , Rs = Porta.1 , E = Porta.0 , Db4 = Portb.0 , Db5 = Portb.1 , Db6 = Portb.2 , Db7 = Portb.3
Initlcd                      'LCD initialisieren

'*** Tonerzeugung mittels Timer0 konfigurieren:
Tccr0a = &B00010010          'OC0A disconnected, Toggle OC0B & Clear Timer on Compare Match
Tccr0b = &B00000001          'Non PWM-Mode, No prescaling
Ocr0a = 16                   'unhrbaren Ton ausgeben - ACHTUNG: 0 verursacht sehr hohe Stromaufnahme

'*** Timer1 einstellen (erzeugt Tonwechsel-Intervalle):
Config Timer1 = Timer , Prescale = 1024
Const Intervall = 480        '480 bei Prescaler 1024/1MHz ergeben ca. 0,5 Sek

'*** Unterprogramme:
Declare Sub Tonaendern()
Declare Sub Startanzeige()
Declare Sub Betriebsanzeige()
Declare Sub Ergebnisanzeige()

'*** Programmstart mit Begrung:
Startanzeige
Wait 2

'*** Hauptschleife:
Do

 '*** Programmablauf ber Stati 0 - 4 steuern:
 Select Case Status
  Case 0 :                   '0 = Programmstart
   Ocr0a = 24                '  -> 20 kHz Ton ausgeben (bei OCR0A=25)
   Betriebsanzeige           '  -> Anzeige "Taste bei Ton!"
   Status = 1                '  -> nchster Status
  Case 1:                    '1 = Warten auf Tastendruck / Ton ausgeben
   Tonaendern                '  -> Frequenz alle 0,5 Sek. erhhen
  Case 2:                    '2 = Taste whrend Tonausgabe gedrckt
   Ergebnisanzeige           '  -> Ergebnis anzeigen
   Status = 3                '  -> nchster Status
   Wait 1                    '  -> Wartezeit wegen Tastenprellens
  Case 3:                    '4 = Ergebnisanzeige
                             '  -> Warten auf erneuten Tastendruck
  Case 4:                    '5 = Taste whrend Ergebnisanzeige gedrckt ...
   Ocr0a = 16                '  -> unhrbaren Ton ausgeben
   Startanzeige              '  -> Startmeldung anzeigen
   Status = 0                '  -> Neustart im nchsten Durchlauf
   Wait 1                    '  -> Wartezeit wegen Tastenprellens
 End Select

 '*** Tasten abfragen:
 Tasten = Pind And &B00011111       'alle Tasten 5 auswerten (D0-D4)
 If Tasten > 0 Then Incr Status       'nchster Programmschritt bei Tastendruck

Loop


'*** Frequenz alle 0,5 Sek. verringern:
Sub Tonaendern
' Die Frequenz ergibt sich aus dem Wert im Timer0-Compare-Register OCR0A
' nach folgender Formel (hherer Wert = niedrigere Frequenz):
' 1.000 Khz (Prozessortakt) / 25 (Startwert) / 2 (Vollwelle) = 20 Khz

 'Sub verlassen, wenn noch keine 0,5 Sek. vorbei:
 If Timer1 < Intervall Then Exit Sub
 'andernfalls ...
 Timer1 = 0                  'Timer1 neu starten
 Incr Ocr0a                  'Frequenz einen Schritt senken
 If Ocr0a = 101 Then Ocr0a = 25       'von 5 KHz wieder auf 20 kHz springen
 'blinkenden Block auf LCD erzeugen:
 Locate 1 , 8
 'Block anzeigen, wenn niederwertigstes Bit gesetzt (ungerader Wert):
 If Ocr0a.0 = 1 Then Lcddata 255 Else Lcd " "
End Sub


'*** Begrung auf LCD:
Sub Startanzeige()
 Cls
 Lcd "H"
 Lcd Chr(239)                'ein 
 Lcd "rtest "
 Locate 2 , 1
 Lcd "  fst'13"
End Sub


'*** Anzeige auf LCD:
Sub Betriebsanzeige()
 Locate 1 , 1
 Lcd "Taste   "
 Locate 2 , 1
 Lcd "bei Ton!"
End Sub


'*** Aktuelle Frequenz anzeigen:
Sub Ergebnisanzeige
 Dim Hz As Word
 Hz = 50000 \ Ocr0a          'Frequenz in Hertz = halber Prozessortakt \ Wert im Compare Register
 Hz = Hz * 10                '...weil der (halbe) Prozessortakt nicht in eine Word-Variable passt
 Locate 1 , 1
 Lcd "Frequenz"
 Locate 2 , 1
 Lcd Hz
 Lcd " Hz "
End Sub